/*
** Module   : XCommonImpl
** Abstract : SAX attributes, reader, writer common fatures iplementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130214 Created initial version. Implementation is limited to the stubs to get the
**                  converted java code compiled.
** 002 CA  20130221 Added Deletable support. This resource doesn't have NEXT/PREV-SIBLING support. 
** 003 OM  20130304 Refactored isValid and isUnknown of WrappedResource to valid and unknown.
** 004 CA  20130603 resourceDelete must return a boolean value.
** 005 EVK 20130903 Made class abstract and move some methods to children. Cosmetic refactoring.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Implementation the common features for XML related Progress objects.
 */
public abstract class XCommonImpl
extends HandleChain
implements XCommon
{
   // Attributes
   /**
    * Gets the value of the XML schema file for elements with no namespaces.
    *
    * @return   The value of the nonamespace-schema-location attribute.
    *           Default value is ""
    */
   @Override
   public character getNonamespaceSchemaLocation()
   {
      return new character("");//override in XDocumentImpl
   }
   
   /**
    * Setting the new value for the nonamespace-schema-location attribute.
    * 
    * @param    location
    *           New value of the nonamespace-schema-location attribute to set.
    */
   @Override
   public void setNonamespaceSchemaLocation(character location)
   {
      //overrided in XDocumentImpl
   }
   
   /**
    * Setting the new value for the nonamespace-schema-location attribute.
    * 
    * @param    location
    *           New value of the nonamespace-schema-location attribute to set.
    */
   @Override
   public void setNonamespaceSchemaLocation(String location)
   {
      setNonamespaceSchemaLocation(new character(location));
   }
   
   // Methods
   /**
    * Adding new match pair for namespace schema and physical file location.
    *
    * @param    namespace
    *           Target namespace of the schema or am empty string or Unknown value.
    * @param    location
    *           Location of the XML schema file.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical addSchemaLocation(String namespace, character location)
   {
      return addSchemaLocation(new character(namespace), location);
   }

   /**
    * Adding new match pair for namespace schema and physical file location.
    *
    * @param    namespace
    *           Target namespace of the schema or am empty string or Unknown value.
    * @param    location
    *           Location of the XML schema file.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical addSchemaLocation(character namespace, String location)
   {
      return addSchemaLocation(namespace, new character(location));
   }

   /**
    * Adding new match pair for namespace schema and physical file location.
    *
    * @param    namespace
    *           Target namespace of the schema or am empty string or Unknown value.
    * @param    location
    *           Location of the XML schema file.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical addSchemaLocation(String namespace, String location)
   {
      return addSchemaLocation(new character(namespace), new character(location));
   }

   /**
    * Adding new match pair for namespace schema and physical file location.
    *
    * @param    namespace
    *           Target namespace of the schema or am empty string or Unknown value.
    * @param    location
    *           Location of the XML schema file.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical addSchemaLocation(character namespace, character location)
   {
      //overrided in XDocumentImpl
      return new logical(true);
   }


   /**
    * To be used only in cases when a setter for a read-only attribute or a not-supported
    * attribute is forced to be implemented by the resource. It displays appropriate error
    * message.
    *
    * @param    attr
    *           The read-only attribute.
    */
   public void readOnlyError(String attr)
   {
      if (attr == null)
      {
         throw new IllegalArgumentException("The attribute is null !");
      }

      final String msg = "**%s is not a settable attribute for %s widget";
      String err = String.format(msg,
                                 attr.toUpperCase(),
                                 type());
      ErrorManager.addError(4052, err, false, true);
   }

   /**
    * Removes the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(character name)
   {
      return removeAttribute(name, new character());
   }
   
   /**
    * Removes the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(String name)
   {
      return removeAttribute(new character(name), new character());
   }
   
   /**
    * Removes the specified attribute.
    *
    * @param    index
    *           The 1-based index of the attribute to remove.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(NumberType index)
   {
      return removeAttribute(index, new character());
   }

   /**
    * Removes the specified attribute.
    *
    * @param    index
    *           The 1-based index of the attribute to remove.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(double index)
   {
      return removeAttribute(new integer(index), new character());
   }

   /**
    * Removes the specified attribute.
    *
    * @param    name
    *           The name of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(String name, String uri)
   {
      return removeAttribute(new character(name), new character(uri));
   }

   /**
    * Removes the specified attribute.
    *
    * @param    name
    *           The name of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(String name, character uri)
   {
      return removeAttribute(new character(name), uri);
   }

   /**
    * Removes the specified attribute.
    *
    * @param    index
    *           The 1-based index of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(double index, character uri)
   {
      return removeAttribute(new integer(index), uri);
   }

   /**
    * Removes the specified attribute.
    *
    * @param
    name
    *           The name of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(character name, String uri)
   {
      return removeAttribute(name, new character(uri));
   }

   /**
    * Removes the specified attribute.
    *
    * @param    index
    *           The 1-based index of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(double index, String uri)
   {
      return removeAttribute(new integer(index), new character(uri));
   }

   /**
    * Removes the specified attribute.
    *
    * @param    index
    *           The 1-based index of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(NumberType index, String uri)
   {
      return removeAttribute(index, new character(uri));
   }
   
   /**
    * Delete the resource.
    * 
    * @return   <code>true</code> if the resource was deleted.
    */
   @Override
   protected boolean resourceDelete()
   {
      return false;
   }

   /**
    * Check if this resource supports the PREV-SIBLING attribute.
    * 
    * @return   Always <code>false</code>.
    */
   @Override
   protected boolean hasPrevSibling()
   {
      return false;
   }
   
   /**
    * Check if this resource supports the NEXT-SIBLING attribute.
    * 
    * @return   Always <code>false</code>.
    */
   @Override
   protected boolean hasNextSibling()
   {
      return false;
   }
}
